@@ -7,7 +7,7 @@ use warnings 'all';
 ###########################################################################
 # METADATA
 our $AUTHORITY = 'cpan:DOUGDUDE';
-our $VERSION   = '0.13';
+our $VERSION   = '0.14';
 
 ###########################################################################
 # MOOSE
@@ -16,7 +16,7 @@ use MooseX::StrictConstructor 0.08;
 
 ###########################################################################
 # MOOSE TYPES
-use Nagios::Plugin::OverHTTP::Library 0.12 qw(
+use Nagios::Plugin::OverHTTP::Library 0.14 qw(
 	Hostname
 	HTTPVerb
 	Path
@@ -31,7 +31,11 @@ use Carp qw(croak);
 use HTTP::Request 5.827;
 use HTTP::Status 5.817 qw(:constants);
 use LWP::UserAgent;
+use Nagios::Plugin::OverHTTP::Formatter::Nagios::Auto;
+use Nagios::Plugin::OverHTTP::Middleware::PerformanceData;
+use Nagios::Plugin::OverHTTP::Parser::Standard;
 use Readonly 1.03;
+use Try::Tiny;
 use URI;
 
 ###########################################################################
@@ -49,6 +53,10 @@ Readonly our $STATUS_WARNING  => 1;
 Readonly our $STATUS_CRITICAL => 2;
 Readonly our $STATUS_UNKNOWN  => 3;
 
+###########################################################################
+# PRIVATE CONSTANTS
+Readonly my $DEFAULT_REQUEST_METHOD => q{GET};
+
 ###########################################################################
 # ATTRIBUTES
 has 'autocorrect_unknown_html' => (
@@ -60,13 +68,22 @@ has 'autocorrect_unknown_html' => (
 
 	default       => 1,
 );
+has 'critical' => (
+	is            => 'rw',
+	isa           => 'HashRef[Str]',
+	documentation => q{Specifies performance levels that result in a }
+	                .q{critical status},
+
+	default       => sub { {} },
+);
 has 'default_status' => (
 	is            => 'rw',
 	isa           => Status,
 	documentation => q{The default status if none specified in the response},
 
 	coerce        => 1,
-	default       => $STATUS_UNKNOWN,
+	default       => $Nagios::Plugin::OverHTTP::Library::STATUS_UNKNOWN,
+	trigger       => sub { shift->_clear_parser },
 );
 has 'hostname' => (
 	is            => 'rw',
@@ -77,15 +94,7 @@ has 'hostname' => (
 	clearer       => '_clear_hostname',
 	lazy          => 1,
 	predicate     => '_has_hostname',
-	trigger       => sub {
-		my ($self) = @_;
-
-		# Clear the state
-		$self->_clear_state;
-
-		# Clear the URL
-		$self->_clear_url;
-	},
+	trigger       => \&_reset_trigger,
 );
 has 'message' => (
 	is            => 'ro',
@@ -97,6 +106,16 @@ has 'message' => (
 	predicate     => 'has_message',
 	traits        => ['NoGetopt'],
 );
+has 'parser' => (
+	is            => 'rw',
+	does          => 'Nagios::Plugin::OverHTTP::Parser',
+	documentation => q{HTTP response parser},
+
+	builder       => '_build_parser',
+	clearer       => '_clear_parser',
+	lazy          => 1,
+	traits        => ['NoGetopt'],
+);
 has 'path' => (
 	is            => 'rw',
 	isa           => Path,
@@ -107,15 +126,7 @@ has 'path' => (
 	coerce        => 1,
 	lazy          => 1,
 	predicate     => '_has_path',
-	trigger       => sub {
-		my ($self) = @_;
-
-		# Clear the state
-		$self->_clear_state;
-
-		# Clear the URL
-		$self->_clear_url;
-	},
+	trigger       => \&_reset_trigger,
 );
 has 'ssl' => (
 	is            => 'rw',
@@ -126,15 +137,7 @@ has 'ssl' => (
 	clearer       => '_clear_ssl',
 	lazy          => 1,
 	predicate     => '_has_ssl',
-	trigger       => sub {
-		my ($self) = @_;
-
-		# Clear the state
-		$self->_clear_state;
-
-		# Clear the URL
-		$self->_clear_url;
-	},
+	trigger       => \&_reset_trigger,
 );
 has 'timeout' => (
 	is            => 'rw',
@@ -188,12 +191,22 @@ has 'verb' => (
 
 	default       => 'GET',
 );
+has 'warning' => (
+	is            => 'rw',
+	isa           => 'HashRef[Str]',
+	documentation => q{Specifies performance levels that result in a }
+	                .q{warning status},
+
+	default       => sub { {} },
+);
 
 ###########################################################################
 # METHODS
 sub check {
 	my ($self) = @_;
 
+	my ($message, $status);
+
 	# Save the current timeout for the useragent
 	my $old_timeout = $self->useragent->timeout;
 
@@ -203,79 +216,40 @@ sub check {
 		$self->useragent->timeout($self->timeout);
 	}
 
-	# Form the HTTP request
-	my $request = HTTP::Request->new($self->verb, $self->url);
-
 	# Get the response of the plugin
-	my $response = $self->useragent->request($request);
+	my $response = try {
+		# Make request
+		$self->request(method => $self->verb, url => $self->url);
+	}
+	catch {
+		# Message is string of the error
+		$message = qq{$_};
+
+		# Status is critical
+		$status = $Nagios::Plugin::OverHTTP::Library::STATUS_CRITICAL;
+
+		# Response undefined
+		undef;
+	};
 
 	# Restore the previous timeout value to the useragent
 	$self->useragent->timeout($old_timeout);
 
-	## no critic (ControlStructures::ProhibitCascadingIfElse)
-	if ($response->code == HTTP_INTERNAL_SERVER_ERROR && $response->message eq 'read timeout') {
-		# Failure due to timeout
-		my $timeout = $self->has_timeout ? $self->timeout : $self->useragent->timeout;
+	if (defined $response) {
+		# Rewrite performance data using default settings
+		$response = Nagios::Plugin::OverHTTP::Middleware::PerformanceData
+			->new(
+				critical_override => $self->critical,
+				warning_override  => $self->warning,
+			)->rewrite($response);
 
-		$self->_set_state($STATUS_CRITICAL, sprintf 'Socket timeout after %d seconds', $timeout);
-		return;
-	}
-	elsif ($response->code == HTTP_INTERNAL_SERVER_ERROR && $response->message =~ m{\(connect: \s timeout\)}msx) {
-		# Failure to connect to the host server
-		$self->_set_state($STATUS_CRITICAL, 'Connection refused ');
-		return;
-	}
-	elsif (HTTP::Status::is_server_error($response->code)) {
-		# There was some type of internal error
-		$self->_set_state($STATUS_CRITICAL, $response->status_line);
-		return;
-	}
-	elsif (!$response->is_success) {
-		# The response was not a success
-		$self->_set_state($STATUS_UNKNOWN, $response->status_line);
-		return;
-	}
+		# Get the parsed message and status from formatter
+		my $formatter = Nagios::Plugin::OverHTTP::Formatter::Nagios::Auto->new(
+			response => $response,
+		);
 
-	# Get the message from the response
-	my $message = $self->_extract_message_from_response($response);
-
-	# Get the status from the response
-	my $status = $self->_extract_status_from_response($response);
-
-	if (!defined $status) {
-		# The status was not found in the response
-		$status = $self->default_status;
-
-		if ($self->autocorrect_unknown_html
-			&& !defined $response->header('X-Nagios-Information')) {
-			# The setting is active to automatically correct unknown HTML
-			if ($message =~ m{\S+\s*[\r\n]+\s*\S+}msx) {
-				# This is a multi-line response.
-				if ($message =~ m{<(?:html|body|head)[^>]*>}imsx) {
-					# This looks like an HTML response. Most likely it is a
-					# response from the server that was intended for a user
-					# to see.
-
-					# This will be searching through the content to find
-					# something to use as the first line.
-
-					# See if a title or h1 can be found
-					my ($title) = $message =~ m{<title[^>]*>(.+?)</title>}imsx;
-					my ($h1   ) = $message =~ m{<h1   [^>]*>(.+?)</h1   >}imsx;
-
-					if (defined $title) {
-						# There was a title, so add it as the first line of the
-						# message
-						$message = sprintf "%s\n%s", $title, $message;
-					}
-					elsif (defined $h1) {
-						# There was a h1, so add it as the first line of the
-						# message
-						$message = sprintf "%s\n%s", $h1, $message;
-					}
-				}
-			}
-		}
+		$message = $formatter->stdout;
+		$status  = $formatter->exit_code;
 	}
 
 	# Set the plugin state
@@ -283,6 +257,47 @@ sub check {
 
 	return;
 }
+sub create_request {
+	my ($self, %args) = @_;
+
+	# Splice the arguments out
+	my ($method, $url) = @args{qw(method url)};
+
+	if (!defined $method) {
+		# Default method since method is not provided
+		$method = $DEFAULT_REQUEST_METHOD;
+	}
+
+	# Just the method and URL are supported
+	return HTTP::Request->new($method, $url);
+}
+sub request {
+	my ($self, @args) = @_;
+
+	# The request is either the only argument or created from the HASH
+	my $request = @args == 1 ? $args[0]
+	                         : $self->create_request(@args)
+	                         ;
+
+	# Get the response of the plugin
+	my $response = $self->useragent->request($request);
+
+	if (!$response->is_success && $response->code == HTTP_INTERNAL_SERVER_ERROR) {
+		# This response likely came directly from LWP::UserAgent
+		if ($response->message eq 'read timeout') {
+			# Make the message explicitly about the timeout
+			croak sprintf 'Socket timeout after %d seconds',
+				$self->useragent->timeout;
+		}
+		elsif ($response->message =~ m{\(connect: \s timeout\)}msx) {
+			# Failure to connect to the host server
+			croak 'Connection refused';
+		}
+	}
+
+	# Return the parsed response
+	return $self->parser->parse($response);
+}
 sub run {
 	my ($self) = @_;
 
@@ -295,45 +310,45 @@ sub run {
 
 ###########################################################################
 # PRIVATE METHODS
-sub _build_hostname {
-	my ($self) = @_;
+sub _build_after_check {
+	my ($self, $attribute) = @_;
 
-	# Build the hostname off the URL
+	# Preform the check
+	$self->check;
+
+	# Return the specified attribute for build
+	return $self->{$attribute};
+}
+sub _build_from_url {
+	my ($self, $attribute) = @_;
+
+	# Populate all fields from the URL
 	$self->_populate_from_url;
 
-	return $self->{hostname};
+	# Return the specified attribute for build
+	return $self->{$attribute};
+}
+sub _build_hostname {
+	return shift->_build_from_url('hostname');
 }
 sub _build_message {
-	my ($self) = @_;
-
-	# Preform the check
-	$self->check;
-
-	return $self->{message};
+	return shift->_build_after_check('message');
 }
 sub _build_path {
+	return shift->_build_from_url('path');
+}
+sub _build_parser {
 	my ($self) = @_;
 
-	# Build the path off the URL
-	$self->_populate_from_url;
-
-	return $self->{path};
+	# Standard parser with the default status
+	return Nagios::Plugin::OverHTTP::Parser::Standard
+		->new(default_status => $self->default_status);
 }
 sub _build_ssl {
-	my ($self) = @_;
-
-	# Build the SSL off the URL
-	$self->_populate_from_url;
-
-	return $self->{ssl};
+	return shift->_build_from_url('ssl');
 }
 sub _build_status {
-	my ($self) = @_;
-
-	# Preform the check
-	$self->check;
-
-	return $self->{status};
+	return shift->_build_after_check('status');
 }
 sub _build_url {
 	my ($self) = @_;
@@ -365,45 +380,6 @@ sub _clear_state {
 	# Nothing useful to return, so chain
 	return $self;
 }
-sub _extract_message_from_response {
-	my ($self, $response) = @_;
-
-	my $message;
-
-	# First priority is the X-Nagios-Information header
-	if (defined $response->header('X-Nagios-Information')) {
-		# Set the message
-		$message = join qq{\n},
-			$response->header('X-Nagios-Information');
-	}
-	else {
-		# Otherwise the message is the body
-		$message = $response->decoded_content;
-	}
-
-	# Return the message
-	return $message;
-}
-sub _extract_status_from_response {
-	my ($self, $response) = @_;
-
-	# First priority is the X-Nagios-Status header
-	my $status = to_Status($response->header('X-Nagios-Status'));
-
-	if (!defined $status && !defined $response->header('X-Nagios-Information')) {
-		# Since X-Status-Information is not present, attempt to extract it
-		# from the body
-		my $message = $response->decoded_content;
-
-		if (my ($inc_status) = $message =~ m{\A ([A-Z]+)\b }msx) {
-			# Attempt to get the status from the first all-caps word
-			$status = to_Status($inc_status);
-		}
-	}
-
-	# Return the status
-	return $status;
-}
 sub _populate_from_url {
 	my ($self) = @_;
 
@@ -426,14 +402,25 @@ sub _populate_from_url {
 	# Nothing useful to return, so chain
 	return $self;
 }
+sub _reset_trigger {
+	my ($self) = @_;
+
+	# Clear the state
+	$self->_clear_state;
+
+	# Clear the generated URL
+	$self->_clear_url;
+
+	return;
+}
 sub _set_state {
 	my ($self, $status, $message) = @_;
 
 	my %status_prefix_map = (
-		$STATUS_OK       => 'OK',
-		$STATUS_WARNING  => 'WARNING',
-		$STATUS_CRITICAL => 'CRITICAL',
-		$STATUS_UNKNOWN  => 'UNKNOWN',
+		$Nagios::Plugin::OverHTTP::Library::STATUS_OK       => 'OK',
+		$Nagios::Plugin::OverHTTP::Library::STATUS_WARNING  => 'WARNING',
+		$Nagios::Plugin::OverHTTP::Library::STATUS_CRITICAL => 'CRITICAL',
+		$Nagios::Plugin::OverHTTP::Library::STATUS_UNKNOWN  => 'UNKNOWN',
 	);
 
 	if ($message !~ m{\A $status_prefix_map{$status}}msx) {
@@ -461,7 +448,7 @@ Nagios::Plugin::OverHTTP - Nagios plugin to check over the HTTP protocol.
 
 =head1 VERSION
 
-Version 0.13
+This documentation refers to L<Nagios::Plugin::OverHTTP> version 0.14
 
 =head1 SYNOPSIS
 
@@ -517,10 +504,13 @@ Arguments should be in the following format on the command line:
   --url http://example.net/check_something
   # Note that quotes may be used, based on your shell environment
 
-  # For bools, like SSL, you would use:
+  # For Booleans, like SSL, you would use:
   --ssl    # Enable SSL
   --no-ssl # Disable SSL
 
+  # For HashRefs, like warning and critical, you would use:
+  --warning name=value --warning name2=value2
+
 =head1 ATTRIBUTES
 
   # Set an attribute
@@ -539,6 +529,14 @@ and the message looks like HTML and has multiple lines. The title of the web
 page will be added to the first line, or the first H1 element will. The default
 for this is on.
 
+=head2 critical
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This is a hash reference specifying different performance names (as the hash
+keys) and what threshold they need to be to result in a critical status. The
+format for the threshold is specified in L</PERFORMANCE THRESHOLD>.
+
 =head2 default_status
 
 B<Added in version 0.09>; be sure to require this version for this feature.
@@ -554,6 +552,13 @@ or a string with the name of the status, like:
 This is the hostname of the remote server. This will automatically be populated
 if L</url> is set.
 
+=head2 parser
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This is a response parser object that does L<Nagios::Plugin::OverHTTP::Parser>.
+By default, it is the L<Nagios::Plugin::OverHTTP::Parser::Standard> parser.
+
 =head2 path
 
 This is the path to the remove Nagios plugin on the remote server. This will
@@ -590,6 +595,14 @@ B<Added in version 0.12>; be sure to require this version for this feature.
 This is the HTTP verb that will be used to make the HTTP request. The default
 value is C<GET>.
 
+=head2 warning
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This is a hash reference specifying different performance names (as the hash
+keys) and what threshold they need to be to result in a warning status. The
+format for the threshold is specified in L</PERFORMANCE THRESHOLD>.
+
 =head1 METHODS
 
 =head2 check
@@ -597,6 +610,36 @@ value is C<GET>.
 This will run the remote check. This is usually not needed, as attempting to
 access the message or status will result in the check being performed.
 
+=head2 create_request
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This will create a L<HTTP::Request> object from the arguments including any
+additional headers added by the plugin. This method takes a HASH as the
+argument with the following keys:
+
+=over 4
+
+=item method
+
+This is the HTTP request method. By default this will be C<GET>.
+
+=item url
+
+B<Required>. This is the URL to request.
+
+=back
+
+=head2 request
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This will return a L<Nagios::Plugin::OverHTTP::Response> object representing
+the plugin response from the server. This method takes either one argument
+which is a L<HTTP::Request> object that will use L</useragent> to make the
+request, or a HASH with the same arguments as L</create_request>. The response
+is parsed with the parser in L</parser>.
+
 =head2 run
 
 This will run the plugin in a standard way. The message will be printed to
@@ -607,6 +650,40 @@ following:
 
   exit $plugin->run;
 
+=head1 PERFORMANCE THRESHOLD
+
+Anywhere a performance threshold is accepted, the threshold value can be in any
+of the following formats (same as listed in
+L<http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT>):
+
+=over 4
+
+=item C<< <number> >>
+
+This will cause an alert if the level is less than zero or greater than
+C<< <number> >>.
+
+=item C<< <number>: >>
+
+This will cause an alert if the level is less than C<< <number> >>.
+
+=item C<< ~:<number> >>
+
+This will cause an alert if the level is greater than C<< <number> >>.
+
+=item C<< <number>:<number2> >>
+
+This will cause an alert if the level is less than C<< <number> >> or greater
+than C<< <number2> >>.
+
+=item C<< @<number>:<number2> >>
+
+This will cause an alert if the level is greater than or equal to
+C<< <number> >> and less than or equal to C<< <number2> >>. This is basically
+the exact opposite of the previous format.
+
+=back
+
 =head1 PROTOCOL
 
 =head2 HTTP STATUS
@@ -615,20 +692,22 @@ The protocol that this plugin uses to communicate with the Nagios plugins is
 unique to my knowledge. If anyone knows another way that plugins are
 communicating over HTTP then let me know.
 
-A request that returns a 5xx status will automatically return as CRITICAL and
-the plugin will display the error code and the status message (this will
-typically result in 500 Internal Server Error).
+A request that returns a C<5xx> status will automatically return as CRITICAL
+and the plugin will display the error code and the status message (this will
+typically result in C<500 Internal Server Error>).
 
-A request that returns a 2xx status will be parsed using the methods listed in
-L</HTTP BODY>.
+A request that returns a C<2xx> status will be parsed using the methods listed
+in L</HTTP BODY> and L</HTTP HEADER>.
 
-Any other status code will cause the plugin to return as UNKNOWN and the plugin
-will display the error code and the status message.
+If the response results is a redirect, the L</useragent> will automatically
+redirect the response and all processing will ultimately be done on the final
+response. Any other status code will cause the plugin to return as UNKNOWN and
+the plugin will display the error code and the status message.
 
 =head2 HTTP BODY
 
 The body of the HTTP response will be the output of the plugin unless the
-header C<X-Nagios-Information> is present. To determine what the status code
+header L</X-Nagios-Information> is present. To determine what the status code
 will be, the following methods are used:
 
 =over 4
@@ -636,23 +715,56 @@ will be, the following methods are used:
 =item 1.
 
 If a the header C<X-Nagios-Status> is present, the value from that is used as
-the output. The content of this header MUST be either the decimal return value
-of the plugin or an all capital letters. The different possibilities for this
-is listed in L</NAGIOS STATUSES>.
+the output. See L</X-Nagios-Status>.
 
 =item 2.
 
-If the header did not conform to proper specifications or was not present, then
-the status will be extracted from the body of the response. The very first set
-of all capital letters is taken from the body and used to determine the result.
-The different possibilities for this is listed in L</NAGIOS STATUSES>
+If the header was not present, then the status will be extracted from the body
+of the response. The very first set of all capital letters is taken from the
+body and used to determine the result. The different possibilities for this is
+listed in L</NAGIOS STATUSES>.
 
 =back
 
-Please note that if the header C<X-Nagios-Information> is present, then the
-status MUST be in the header C<X-Nagios-Status> as described above. The status
-will not be extracted from any text. The C<X-Nagios-Information> header support
-was added in version 0.12.
+=head2 HTTP HEADER
+
+The following HTTP headers have special meanings:
+
+=head3 C<< X-Nagios-Information >>
+
+B<Added in version 0.12>; be sure to require this version for this feature.
+
+If this header is present, then the content of this header will be used as the
+message for the plugin. Note: B<the body will not be parsed>. This is meant as
+an indication that the Nagios output is solely contained in the headers. This
+MUST contain the message ONLY. If this header appears multiple times, each
+instance is appended together with line breaks in the same order for multiline
+plugin output support.
+
+  X-Nagios-Information: Connection to database succeeded
+  X-Nagios-Information: 'www'@'localhost'
+
+=head3 C<< X-Nagios-Performance >>
+
+B<Added in version 0.14>; be sure to require this version for this feature.
+
+This header specifies various performance data from the plugin. This will add
+performance to the list of any data collected from the response body as
+specified in L</HTTP BODY>. Many performance data may be contained in a single
+header seperated by spaces any many headers may be specified.
+
+  X-Nagios-Performance: 'connect time'=0.0012s
+
+=head3 C<< X-Nagios-Status >>
+
+This header specifies the status. When this header is specified, then this is
+will override any other location where the status can come from. The content of
+this header MUST be either the decimal return value of the plugin or the status
+name in all capital letters. The different possibilities for this is listed in
+L</NAGIOS STATUSES>. If the header appears more than once, the first occurance
+is used.
+
+  X-Nagios-Status: OK
 
 =head2 NAGIOS STATUSES
 
@@ -682,18 +794,18 @@ The following is an example of a simple bootstrapping of a plugin on a remote
 server.
 
   #!/usr/bin/env perl
-  
+
   use strict;
   use warnings;
-  
+
   my $output = qx{/usr/local/libexec/nagios/check_users2 -w 100 -c 500};
-  
+
   my $status = $? > 0 ? $? >> 8 : 3;
-  
-  printf "X-Nagios-Header: %d\n", $status;
+
+  printf "X-Nagios-Status: %d\n", $status;
   print  "Content-Type: text/plain\n\n";
   print  $output if $output;
-  
+
   exit 0;
 
 =head1 DEPENDENCIES
@@ -716,6 +828,8 @@ server.
 
 =item * L<Readonly> 1.03
 
+=item * L<Try::Tiny>
+
 =item * L<URI>
 
 =item * L<namespace::clean> 0.04
@@ -731,7 +845,10 @@ Douglas Christopher Wilson, C<< <doug at somethingdoug.com> >>
 =over
 
 =item * Alex Wollangk contributed the idea and code for the
-C<X-Nagios-Information> header.
+L</X-Nagios-Information> header.
+
+=item * Peter van Eijk pushed me to get performance data handling
+implemented.
 
 =back
 
@@ -773,7 +890,7 @@ L<http://search.cpan.org/dist/Nagios-Plugin-OverHTTP/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2009 Douglas Christopher Wilson, all rights reserved.
+Copyright 2009-2010 Douglas Christopher Wilson, all rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of either:
